new;
library pgraph;

/*
** Modele GBM multidimensionnel
**
** dX_i(t) = \mu_i * X_i(t) * dt + \sigma_i * X_i(t) * dW_i(t), i = 1,...,n
**
** E[W_i(t)W_j(t)] = rho_{i,j}
**
** 
*/

proc (1) = simulate_mGBM(x0,mu,sigma,rho,t);
  local Nt,Nx,x,Pchol,i,dt,k1,k2,u;

  Nt = rows(t);
  Nx = rows(x0);

  x = zeros(Nt,Nx);
  x0 = x0';
  mu = mu';
  sigma = sigma';
  Pchol = chol(rho);                // Decomposition Cholesky de la matrice de correlation

  i = 1;
  do until i > Nt;

    if i == 1;
      dt = t[1];
    else;
      dt = t[i] - t[i-1];
    endif;

    k1 = (mu-0.5*sigma^2) * dt;
    k2 = sigma * sqrt(dt);
    u = rndn(1,nx)*Pchol;

    x0 = x0 .* exp( k1 + k2 .* u);
    x[i,.] = x0;

    i = i + 1;
  endo;

  retp(x);
endp;

let x0 = 100 50 75;            
let mu = 0.05 0.07 0.08;
let sigma = 0.15 0.25 0.05;

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1};

rho = xpnd(rho);  // transforme le vecteur 'vech(rho)' en matrice de correlation

t = seqa(0,1/365,5*365+1);

x = simulate_mGBM(x0,mu,sigma,rho,t);

graphset;
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3; _plwidth = 5;
  title("\214GBM Simulation");
  xlabel("\214time");
  xtics(0,5,1,12);
  xy(t,x);


/* 
** Si nous voulons obtenir non pas une seule trajectoire simulee du processus 3D
** mais plusieurs trajectoires, nous pouvons utiliser une boucle
**
*/

Ns = 100;
Nt = rows(t);

x1 = zeros(Nt,Ns); x2 = x1; x3 = x1;

t0 = hsec;

for i (1,Ns,1);
  x = simulate_mGBM(x0,mu,sigma,rho,t);
  x1[.,i] = x[.,1]; 
  x2[.,i] = x[.,2]; 
  x3[.,i] = x[.,3]; 
endfor;

ct = (hsec-t0)/100;

print ftos(ct,"Computational time : %f seconds.",5,2);

graphset;
  _pltype = 3;
  xy(t,x1[.,1:100]);   // Graphe des 100 premieres trajectoires de X_1(t)
