new;
library pgraph;

cls;

/*
** Modele GBM multidimensionnel
**
** dX_i(t) = \mu_i * X_i(t) * dt + \sigma_i * X_i(t) * dW_i(t), i = 1,...,n
**
** E[W_i(t)W_j(t)] = rho_{i,j}
**
** 
*/

proc (1) = matrix_simulate_mGBM(x0,mu,sigma,rho,t);
  local Nt,Nx,x,Pchol,i,dt,k1,k2,u;

  Nt = rows(t);
  Nx = rows(x0);

  x = zeros(Nt,Nx);
  x0 = x0';
  mu = mu';
  sigma = sigma';
  Pchol = chol(rho);                // Decomposition Cholesky de la matrice de correlation

  i = 1;
  do until i > Nt;

    if i == 1;
      dt = t[1];
    else;
      dt = t[i] - t[i-1];
    endif;

    k1 = (mu-0.5*sigma^2) * dt;
    k2 = sigma * sqrt(dt);
    u = rndn(1,nx)*Pchol;

    x0 = x0 .* exp( k1 + k2 .* u);
    x[i,.] = x0;

    i = i + 1;
  endo;

  retp(x);
endp;

proc (1) = array_simulate_mGBM(x0,mu,sigma,rho,t,Ns);
  local Nt,Nx,x,Pchol,i,dt,k1,k2,u;

  Nt = rows(t);
  Nx = rows(x0);

  x = arrayinit(Nt|Ns|Nx,0);
  x0 = x0';
  mu = mu';
  sigma = sigma';
  Pchol = chol(rho);                // Decomposition Cholesky de la matrice de correlation

  i = 1;
  do until i > Nt;

    if i == 1;
      dt = t[1];
    else;
      dt = t[i] - t[i-1];
    endif;

    k1 = (mu-0.5*sigma^2) * dt;
    k2 = sigma * sqrt(dt);
    u = rndn(ns,nx)*Pchol;

    x0 = x0 .* exp( k1 + k2 .* u);
    x[i,.,.] = x0;

    i = i + 1;
  endo;

  retp(x);
endp;

let x0 = 100 50 75;            
let mu = 0.05 0.07 0.08;
let sigma = 0.15 0.25 0.05;

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1};

rho = xpnd(rho);  // transforme le vecteur 'vech(rho)' en matrice de correlation

dt = 365;
t = seqa(0,1/dt,5*dt+1);

/* 
** Verifions tout d'abord que les procedures array_simulate_GBM
** et matrix_simulate_GBM donnent les memes resultats lorsque 
** le nombre de trajectoires est fixe a 1.
*/

rndseed 123;
x = array_simulate_mGBM(x0,mu,sigma,rho,t,1);
x = getmatrix(atranspose(x,2|1|3),1);

rndseed 123;
y = matrix_simulate_mGBM(x0,mu,sigma,rho,t);

err = maxc(maxc(abs(x-y)));
print ftos(err,"Maximal absolute error = %lE",10,5);


/* 
** Comparons maintenant les temps de calcul
*/

let ns = 1 2 3 10 100;

print;
print "Computational time for matrix_simulate_mGBM";

j = 1;
do until j > rows(ns);

  t0 = hsec;
  for i (1,ns[j],1);
    y = matrix_simulate_mGBM(x0,mu,sigma,rho,t);
  endfor;
  print /flush ftos(ns[j],"for %lf simulations",1,0)$+ftos((hsec-t0)/100," = %lf seconds",5,2);

  j = j + 1;
endo;

print;
print "Computational time for array_simulate_mGBM";

j = 1;
do until j > rows(ns);

  t0 = hsec;
  x = array_simulate_mGBM(x0,mu,sigma,rho,t,ns[j]);
  print /flush ftos(ns[j],"for %lf simulations",1,0)$+ftos((hsec-t0)/100," = %lf seconds",5,2);
  j = j + 1;
endo;

Ns = 100;

x = array_simulate_mGBM(x0,mu,sigma,rho,t,Ns);

x = atranspose(x,3|1|2);
x1 = getmatrix(x,1);              /* Matrice Nt * 100 : la colonne i correspond a la i-ieme trajectoire */
x2 = getmatrix(x,2);
x3 = getmatrix(x,3);

