/* 
** This will do the between regression in table 2.1
** AND, as an added benefit,  get the book's results 
**
** It uses twoway (could have used between) to get group means
** and then uses formulas i Greene on page 619
**
** The program then uses the P transformation and finally the
** transformation suggested by Baltagi on p 16
*/

new;
#include twoway.src;

msk = zeros(1,1)~ones(1,4);
let fmt[5,3] = "*.*s " 8 8 "*.*lg" 12 4 "*.*lg" 12 4 "*.*lg" 12 4 "*.*le" 12 2;
msk1 = zeros(1,1)~ones(1,1);
let fmt1[2,3] = "*.*s " 8 8 "*.*lg" 12 4;

N = 10;    @ number of individuals @
T = 20;    @ length of time period @
NT = N*T;
K = 2;     @ number of independents @

load Data[NT,K+1] = grunfeld.txt;


/* 
**   NOTE THAT THE DEPENDENT VARIABLE IS THE FIRST COLUMN IN Data
**
**   This is not required by the subroutine WITHIN but it makes book keeping
**   easier. Thus the independents are columns 2 to K+1
**
**   If the data is not organized in this way, you could always rearrange it
**   after it is entered. Thus, if column J is the dependent and cols 1:J-1, and
**   J+1:K+1 are the independents, simple code
**
**   y = Data[.,J];
**   x = Data[.,1:J-1 J+1:K+1];
**
**   and call WITHIN and the rest will work.
*/

y0 = Data[.,1];         
x0 = Data[.,2:k+1];

names = "   Value"|"   Stock";
namesc = "Constant"|names;

{z,m1,m2,m3} = twoway(y0~x0,N,T,(1|0|0));  @ we want group means in m1 @

x = m1[.,2:3]-m3[2:3]';                    @ our x matrix @
y = m1[.,1]-m3[1];                         @ our y matrix @


/* 
**   The P-transformation in Baltagi actually removes the overall mean
**   from each of the group means and stacks the resulting matrix t times
**
**   The way I do it is easier, but the result is the same
*/

sxx = t*x'x;          @ Sum of squares: Green, middle of page 619 @
sxy = t*x'y;          @ Sum of squares: Green, middle of page 619 @

b = solpd(sxy,sxx);             @ 14-13 in Greene @
A = m3[1]-m3[2:3]'*b;           @ the constant @
u = m1[.,1]-A-m1[.,2:3]*b;      @ residuals @
s2 = T*u'u/(N-k-1);             @ residual variance @
vcv = s2*invpd(Sxx);            @ vcv matrix @
se = sqrt(diag(vcv));           @ standard errors @

tF = b./se;                           /* t-stats */
sgF = 2*cdftc(abs(tF),(N-3));         /* Significance levels */

va = s2/NT + m3[2:3]'*vcv*m3[2:3];    /* var(ALFA) */
sea = sqrt(va);                       /* standard error of ALFA */
t_a = A/sea;                          /* t-stat for ALFA */
sga = 2*cdftc(abs(t_a),(N-3));        /* signif level for ALFA */
  
r2 = 1-T*u'u/(T*y'y);                  /* R2 */


/* print the final estimates */
  
format 16,6; print; print;
f0 = "The Between regression (using transposes):";
$f0;
"*********************************************************";
"Degrees of freedom           : ";; (N-3);
"Residual sum of squares      : ";; T*u'u;
"Residual variance            : ";; s2;
"R-squared                    : ";; r2;
"*********************************************************";
"Variable         coef     std.err      t-stat      Signif";
f0 = printfm(namesc~(A|b)~(sea|se)~(t_a|tF)~(sga|sgF),msk,fmt);
"*********************************************************";

/* Now do it by brute force and compare */

P = eye(N) .*. (ones(T,T)/T);       /* The P transformation */
x = ones(rows(P),1)~P*x0;
y = P*y0;

b = y/x;
e = y - x*b;
s2 = e'e/(N-K-1);
ss = s2;
se = sqrt(diag(s2*invpd(x'x)));
tF = b./se;                         /* t-stats */
sgF = 2*cdftc(abs(tF),(N-3));       /* Significance levels */

r2 = 1-e'e/(y'y-NT*meanc(y)^2);     /* R2 */

/* print the final estimates */
  
format 16,6;
print;print;
f0 = "The between regression (using brute force):";
$f0;
"*********************************************************";
"Degrees of freedom           : ";; (N-3);
"Residual sum of squares      : ";; e'e;
"Residual variance            : ";; s2;
"R-squared                    : ";; r2;
"*********************************************************";
"Variable         coef     std.err      t-stat      Signif";
f0 = printfm(namesc~(b)~(se)~(tF)~(sgF),msk,fmt);
"**********************************************************";


/* Finally we can do it as suggested by Baltagi in (2.26) */

y = m1[.,1]*sqrt(T);
x = (ones(N,1)~m1[.,2:3])*sqrt(T);

b = y/x;
e = y - x*b;
s2 = e'e/(N-K-1);
se = sqrt(diag(s2*invpd(x'x)));
tF = b./se;                              /* t-stats */
sgF = 2*cdftc(abs(tF),(N-K-1));          /* Significance levels */

r2 = 1-e'e/(y'y - N*meanc(y)^2);         /* R2 */

format 16,6;
print;print;
f0 = "The between regression (using Baltagi's 2.26):";
$f0;
"*********************************************************";
"Degrees of freedom           : ";; (N-3);
"Residual sum of squares      : ";; e'e;
"Residual variance            : ";; s2;
"R-squared                    : ";; r2;
"*********************************************************";
"Variable         coef     std.err      t-stat      Signif";
f0 = printfm(namesc~(b)~(se)~(tF)~(sgF),msk,fmt);
"**********************************************************";
